@@ -50,27 +50,58 @@
 #include "glib.h"
 #include "galias.h"
 
+#define G_NSEC_PER_SEC 1000000000
+
+#if defined(HAVE_CLOCK_GETTIME) && defined(HAVE_MONOTONIC_CLOCK) 
+#define USE_CLOCK_GETTIME 1
+#endif
 
 struct _GTimer
 {
 #ifdef G_OS_WIN32
   guint64 start;
   guint64 end;
-#else /* !G_OS_WIN32 */
+#elif USE_CLOCK_GETTIME 
+  struct timespec start;
+  struct timespec end;
+  gint clock;
+#else /* uses gettimeofday */
   struct timeval start;
   struct timeval end;
-#endif /* !G_OS_WIN32 */
+#endif 
 
   guint active : 1;
 };
 
 #ifdef G_OS_WIN32
-#  define GETTIME(v) \
+#  define GETTIME(v)				\
      GetSystemTimeAsFileTime ((FILETIME *)&v)
-#else /* !G_OS_WIN32 */
-#  define GETTIME(v) \
+#elif USE_CLOCK_GETTIME
+#  define GETTIME(v)				\
+     clock_gettime (posix_clock, &v)
+#else
+#  define GETTIME(v)				\
      gettimeofday (&v, NULL)
-#endif /* !G_OS_WIN32 */
+#endif
+
+#ifdef USE_CLOCK_GETTIME
+static gint posix_clock = 0;
+
+static void
+init_posix_clock (void)
+{
+  static gboolean initialized = FALSE;
+
+  if (!initialized)
+    {
+      initialized = TRUE;
+      if (sysconf (_SC_MONOTONIC_CLOCK) >= 0)
+	posix_clock = CLOCK_MONOTONIC;
+      else
+	posix_clock = CLOCK_REALTIME;
+    }
+}
+#endif
 
 GTimer*
 g_timer_new (void)
@@ -80,6 +111,10 @@ g_timer_new (void)
   timer = g_new (GTimer, 1);
   timer->active = TRUE;
 
+#ifdef USE_CLOCK_GETTIME
+  init_posix_clock ();
+#endif
+
   GETTIME (timer->start);
 
   return timer;
@@ -126,9 +161,11 @@ g_timer_continue (GTimer *timer)
 {
 #ifdef G_OS_WIN32
   guint64 elapsed;
+#elif USE_CLOCK_GETTIME
+  struct timespec elapsed;
 #else
   struct timeval elapsed;
-#endif /* G_OS_WIN32 */
+#endif
 
   g_return_if_fail (timer != NULL);
   g_return_if_fail (timer->active == FALSE);
@@ -146,7 +183,29 @@ g_timer_continue (GTimer *timer)
 
   timer->start -= elapsed;
 
-#else /* !G_OS_WIN32 */
+#elif USE_CLOCK_GETTIME
+
+  if (timer->start.tv_nsec > timer->end.tv_nsec)
+    {
+      timer->end.tv_nsec += G_NSEC_PER_SEC;
+      timer->end.tv_sec--;
+    }
+
+  elapsed.tv_nsec = timer->end.tv_nsec - timer->start.tv_nsec;
+  elapsed.tv_sec = timer->end.tv_sec - timer->start.tv_sec;
+
+  GETTIME (timer->start);
+
+  if (timer->start.tv_nsec < elapsed.tv_nsec)
+    {
+      timer->start.tv_nsec += G_NSEC_PER_SEC;
+      timer->start.tv_sec--;
+    }
+
+  timer->start.tv_nsec -= elapsed.tv_nsec;
+  timer->start.tv_sec -= elapsed.tv_sec;
+
+#else
 
   if (timer->start.tv_usec > timer->end.tv_usec)
     {
@@ -180,9 +239,11 @@ g_timer_elapsed (GTimer *timer,
   gdouble total;
 #ifdef G_OS_WIN32
   gint64 elapsed;
+#elif USE_CLOCK_GETTIME
+  struct timespec elapsed;
 #else
   struct timeval elapsed;
-#endif /* G_OS_WIN32 */
+#endif 
 
   g_return_val_if_fail (timer != NULL, 0);
 
@@ -196,9 +257,33 @@ g_timer_elapsed (GTimer *timer,
 
   if (microseconds)
     *microseconds = (elapsed / 10) % 1000000;
-#else /* !G_OS_WIN32 */
+#elif HAVE_CLOCK_GETTIME
   if (timer->active)
-    gettimeofday (&timer->end, NULL);
+    GETTIME (timer->end);
+
+  if (timer->start.tv_nsec > timer->end.tv_nsec)
+    {
+      timer->end.tv_nsec += G_NSEC_PER_SEC;
+      timer->end.tv_sec--;
+    }
+
+  elapsed.tv_nsec = timer->end.tv_nsec - timer->start.tv_nsec;
+  elapsed.tv_sec = timer->end.tv_sec - timer->start.tv_sec;
+
+  total = elapsed.tv_sec + ((gdouble) elapsed.tv_nsec / (gdouble) G_NSEC_PER_SEC);
+  if (total < 0)
+    {
+      total = 0;
+
+      if (microseconds)
+	*microseconds = 0;
+    }
+  else if (microseconds)
+    *microseconds = elapsed.tv_nsec / 1000;
+
+#else
+  if (timer->active)
+    GETTIME (timer->end);
 
   if (timer->start.tv_usec > timer->end.tv_usec)
     {
@@ -209,7 +294,7 @@ g_timer_elapsed (GTimer *timer,
   elapsed.tv_usec = timer->end.tv_usec - timer->start.tv_usec;
   elapsed.tv_sec = timer->end.tv_sec - timer->start.tv_sec;
 
-  total = elapsed.tv_sec + ((gdouble) elapsed.tv_usec / 1e6);
+  total = elapsed.tv_sec + ((gdouble) elapsed.tv_usec / (gdouble) G_USEC_PER_SEC);
   if (total < 0)
     {
       total = 0;
@@ -220,7 +305,7 @@ g_timer_elapsed (GTimer *timer,
   else if (microseconds)
     *microseconds = elapsed.tv_usec;
 
-#endif /* !G_OS_WIN32 */
+#endif
 
   return total;
 }
@@ -238,6 +323,14 @@ g_usleep (gulong microseconds)
   while (nanosleep (&request, &remaining) == -1 && errno == EINTR)
     request = remaining;
 # else /* !HAVE_NANOSLEEP */
+#  ifdef HAVE_NSLEEP
+  /* on AIX, nsleep is analogous to nanosleep */
+  struct timespec request, remaining;
+  request.tv_sec = microseconds / G_USEC_PER_SEC;
+  request.tv_nsec = 1000 * (microseconds % G_USEC_PER_SEC);
+  while (nsleep (&request, &remaining) == -1 && errno == EINTR)
+    request = remaining;
+#  else /* !HAVE_NSLEEP */
   if (g_thread_supported ())
     {
       static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
@@ -270,6 +363,7 @@ g_usleep (gulong microseconds)
       tv.tv_usec = microseconds % G_USEC_PER_SEC;
       select(0, NULL, NULL, NULL, &tv);
     }
+#  endif /* !HAVE_NSLEEP */
 # endif /* !HAVE_NANOSLEEP */
 #endif /* !G_OS_WIN32 */
 }
@@ -333,7 +427,7 @@ mktime_utc (struct tm *tm)
   retval += (tm->tm_year - 68) / 4;
   retval += days_before[tm->tm_mon] + tm->tm_mday - 1;
   
-  if (tm->tm_year % 4 == 2 && tm->tm_mon < 2)
+  if (tm->tm_year % 4 == 0 && tm->tm_mon < 2)
     retval -= 1;
   
   retval = ((((retval * 24) + tm->tm_hour) * 60) + tm->tm_min) * 60 + tm->tm_sec;
